guild_bank_stats = {}
guild_bank_stats.running = false
guild_bank_stats.data = {}
guild_bank_stats.data.transactions = {}
guild_bank_stats.data.page_nr = 1
guild_bank_stats.data.players = {}
guild_bank_stats.data.update_needed = true
guild_bank_stats.dlg = {}

function guild_bank_stats.f_sum(n)
  local formatted = n
  local k
  while true do  
		formatted, k = string.gsub(formatted, "^(-?%d+)(%d%d%d)", '%1 %2')
		if (k==0) then
      break
    end
  end
  return formatted
end

function guild_bank_stats.p(s)
	print("\1273388ffBank Stats:\127ffffff "..s)
end

function guild_bank_stats.transaction_exists(t)
	for _,tt in ipairs(guild_bank_stats.data.transactions) do
		if t.timestamp == tt.timestamp and t.charname == tt.charname and t.current == tt.current  then 
			return true
		end
	end

	return false
end

function guild_bank_stats.process_page_event(data)
	if data ~= nil and type(data) == "table" and #data > 0 then
		if guild_bank_stats.data.page_nr % 50 == 0 then
			guild_bank_stats.p("Still getting pages ("..guild_bank_stats.data.page_nr..")")
		end
		for _,transaction in ipairs(data) do 
			if guild_bank_stats.transaction_exists(transaction) then
				guild_bank_stats.finish_generate() -- stop fetching more pages
				return
			else
				table.insert(guild_bank_stats.data.transactions, transaction)
			end
		end
		guild_bank_stats.data.page_nr = guild_bank_stats.data.page_nr + 1 
		Guild.getbanklogpage(guild_bank_stats.data.page_nr)
	else
		guild_bank_stats.finish_generate()
	end
end

function guild_bank_stats:OnEvent(event_name, data)
	if event_name == "GUILD_BANK_LOG"  then
		if guild_bank_stats.running == true then
			guild_bank_stats.process_page_event(data)
		end
	elseif event_name == "GUILD_BALANCE_UPDATED" then
		guild_bank_stats.data.update_needed = true
	end
end

function guild_bank_stats.start_generate()
	Guild.getbanklogpage(1)
end

function guild_bank_stats.get_player(name)
	for i,player in ipairs(guild_bank_stats.data.players) do
		if player.name == name then 
			return player
		end
	end

	return nil
end

function guild_bank_stats.finish_generate()

	table.sort(guild_bank_stats.data.transactions, function(a,b) return (a.timestamp > b.timestamp) end)

	-- clear player stats table
	for k in pairs (guild_bank_stats.data.players) do
		guild_bank_stats.data.players[k] = nil
	end
	
	-- parse data
	local players = guild_bank_stats.data.players
	for _,transaction in ipairs(guild_bank_stats.data.transactions) do
		local player_name = transaction.charname
		local sum = transaction.current - transaction.previous
		local player = guild_bank_stats.get_player(player_name)
		if player == nil then
			player = {name = player_name, total = 0, transactions = {}, deposits = {}, withdraws = {}}
			table.insert(players, player)
		end
		if sum > 0 then
			table.insert(player.deposits, sum)
		else
			table.insert(player.withdraws, sum)
		end
		player.total = player.total + sum
		table.insert(player.transactions, transaction)
	end

	-- continue processing with these
	guild_bank_stats.order()
	guild_bank_stats.running = false
	guild_bank_stats.data.update_needed = false
	guild_bank_stats.create_summary()

end

function guild_bank_stats.order()
	-- order players by total
	table.sort(guild_bank_stats.data.players, function(a,b) return (a.total > b.total) end)

	-- order deposits and withdraws for each player
	for i,player in ipairs(guild_bank_stats.data.players) do
		table.sort(player.deposits, function(a,b) return (a < b) end)
		table.sort(player.withdraws, function(a,b) return (a < b) end)
	end
end

function guild_bank_stats.sum( t )
  local sum = 0
  local count= 0

  for k,v in pairs(t) do
		if type(v) == 'number' then
      sum = sum + v
      count = count + 1
    end
  end

  return sum
end

function guild_bank_stats.mean( t )
  local sum = 0
  local count= 0

  for k,v in pairs(t) do
		if type(v) == 'number' then
      sum = sum + v
      count = count + 1
    end
  end

  if count == 0 then return 0 end

  return math.floor((sum / count))
end

function guild_bank_stats.median( t )
	if #t == 0 then return 0 end

	-- If we have an even number of table elements or odd.
	if math.fmod(#t,2) == 0 then
		-- return mean value of middle two elements
		return math.floor( ( t[#t/2] + t[(#t/2)+1] ) / 2 )
	else
		-- return middle element
		return t[math.ceil(#t/2)]
	end
end

function guild_bank_stats.create_summary()

	for i,player in ipairs(guild_bank_stats.data.players) do
		local total_color = "\12700ff00"
		if player.total < 0 then
			total_color = "\127ff0000"
		end
	end

	-- STATS MATRIX
	local stats_matrix = iup.matrix {numcol=10, numlin=#guild_bank_stats.data.players, expand="YES", markmode="LIN"}
	stats_matrix:setcell(0,1,"Player")
	stats_matrix:setcell(0,2,"Total")
	stats_matrix:setcell(0,3,"D Count")
	stats_matrix:setcell(0,4,"D Total")
	stats_matrix:setcell(0,5,"D Median")
	stats_matrix:setcell(0,6,"D Mean")
	stats_matrix:setcell(0,7,"W Count")
	stats_matrix:setcell(0,8,"W Total")
	stats_matrix:setcell(0,9,"W Median")
	stats_matrix:setcell(0,10,"W Mean")
	for i,player in ipairs(guild_bank_stats.data.players) do
		local total_color = "\12700FF00"
		if player.total < 0 then
			total_color = "\127FF0000"
		end
		stats_matrix:setcell(i, 1, player.name)
		stats_matrix:setcell(i, 2, total_color..guild_bank_stats.f_sum(player.total))
		stats_matrix:setcell(i, 3, #player.deposits)
		stats_matrix:setcell(i, 4, guild_bank_stats.f_sum(guild_bank_stats.sum(player.deposits)))
		stats_matrix:setcell(i, 5, guild_bank_stats.f_sum(guild_bank_stats.median(player.deposits)))
		stats_matrix:setcell(i, 6, guild_bank_stats.f_sum(guild_bank_stats.mean(player.deposits)))
		stats_matrix:setcell(i, 7, #player.withdraws)
		stats_matrix:setcell(i, 8, guild_bank_stats.f_sum(guild_bank_stats.sum(player.withdraws)))
		stats_matrix:setcell(i, 9, guild_bank_stats.f_sum(guild_bank_stats.median(player.withdraws)))
		stats_matrix:setcell(i, 10, guild_bank_stats.f_sum(guild_bank_stats.mean(player.withdraws)))
	end
	

	-- LOG PAGES MATRIX
	local log_pages_matrix = iup.matrix {numcol=5, numlin=#guild_bank_stats.data.transactions, expand="YES", markmode="LIN"}
	log_pages_matrix:setcell(0,1,"Date")
	log_pages_matrix:setcell(0,2,"Name")
	log_pages_matrix:setcell(0,3,"Sum")
	log_pages_matrix:setcell(0,4,"Balance")
	log_pages_matrix:setcell(0,5,"Description")
	local i = 1
	for _,transaction in ipairs(guild_bank_stats.data.transactions) do
		local sum_color = "\12700FF00"
		local sum = transaction.current - transaction.previous
		if sum < 0 then
			sum_color = "\127FF0000"
		end
		log_pages_matrix:setcell(i, 1, os.date("%Y-%m-%d %H:%M",transaction.timestamp))
		log_pages_matrix:setcell(i, 2, transaction.charname)
		log_pages_matrix:setcell(i, 3, sum_color..guild_bank_stats.f_sum(sum))
		log_pages_matrix:setcell(i, 4, guild_bank_stats.f_sum(transaction.current))
		log_pages_matrix:setcell(i, 5, transaction.description)
		i = i + 1
	end


	-- DIALOG
	local close_btn = iup.button{title="Close",expand="HORIZONTAL"}
	local button_box = iup.hbox {close_btn}
	local tab_a_box = iup.vbox {iup.fill{size=10}, log_pages_matrix}
	local tab_b_box = iup.vbox {iup.fill{size=10}, stats_matrix}
	tab_a_box.tabtitle = "Bank Logs"
	tab_b_box.tabtitle = "Bank Stats"
	local tabs = iup.pda_root_tabs{tab_a_box, tab_b_box}
	local dialog_vbox = iup.vbox {tabs,button_box}
	guild_bank_stats.dlg = iup.dialog{dialog_vbox; title="Guild Bank Browser",size="%95x%90",topmost="YES",expand="YES",bgcolor="98 178 217"}
	function close_btn:action ()
		guild_bank_stats.dlg:hide()
	end
	guild_bank_stats.dlg:show()

end

function guild_bank_stats.print()
	if guild_bank_stats.data.update_needed == true then
		guild_bank_stats.p("Getting pages... Please wait...")
		guild_bank_stats.running = true
		guild_bank_stats.start_generate()
	else
		guild_bank_stats.create_summary()      
	end
end

RegisterEvent(guild_bank_stats, "GUILD_BANK_LOG")
RegisterEvent(guild_bank_stats, "GUILD_BALANCE_UPDATED")

RegisterUserCommand("bankstats", guild_bank_stats.print)
